"""FlowHive Agent - GPU-aware task execution agent.

Copyright (c) 2025 Dramwig

This program is free software: you can redistribute it and/or modify
it under the terms of the FlowHive Agent Non-Commercial Copyleft License v1.0.
See LICENSE file for details.

Commercial use requires a separate commercial license.
"""

__version__ = "0.1.0"

from .core import (
    TaskManager,
    Task,
    TaskStatus,
    CommandGroup,
    GPUMonitor,
    GPUStats,
    ProcessUsage,
    GPUService,
)

__all__ = [
    "__version__",
    "TaskManager",
    "Task",
    "TaskStatus",
    "CommandGroup",
    "GPUMonitor",
    "GPUStats",
    "ProcessUsage",
    "GPUService",
]